function y=dsbesselj(nu,Z)
%DSBESSELJ Derivative of the spherical Bessel function of the first kind.
%   Y = BESSELY(NU,Z) is the derivative of the spherical Bessel function 
%   of the first kind, j_nu(Z).
%   The order NU need not be an integer, but must be real.
%   The argument Z can be complex.  The result is real where Z is positive.
%
%   If NU and Z are arrays of the same size, the result is also that size.
%   If either input is a scalar, it is expanded to the other input's size.
%   If one input is a row vector and the other is a column vector, the
%   result is a two-dimensional table of function values.
%
% Author: Stephane BALAC, FOTON UMR CNRS 6082, University of Rennes, France

js1=sbesselj(nu-1,Z);
js2=sbesselj(nu+1,Z);
y=(nu.*js1-(nu+1).*js2)./(2*nu+1);
end


