%ELLRES COMPUTES THE MODE INDEX AT RESONANCE
%   ELLRES computes, for a given wavelength, the values of the mode index
%   ell for which a resonance occurs
%
% Author: Stephane BALAC, FOTON UMR CNRS 6082, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 30 January 2014
%
% Copyright or  or Copr. Stephane BALAC (30 January 2014)
%
% This software is governed by the CeCILL-C license under French law 
% and abiding by the rules of distribution of free software.
% You can  use, modify and/ or redistribute the software under the terms of
% the CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
% URL "http://www.cecill.info". 
% 
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economic rights,  and the successive licensors  have only  limited
% liability. 
% 
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or 
% data to be ensured and,  more generally, to use and operate it in the 
% same conditions as regards security. 
% 
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL-C license and that you accept its terms.

clear 
close all
I=sqrt(-1);
cc=299792458;  % Speed of light
expert_mode=0; % set to 1 for using the programm in expert mode
%
% Data reading 
%
R=input('Micro-sphere radius [micro-m] = ');
R=R*1e-6;
N=input('Optical index of the micro-sphere = ');
lambda=input('Wavelength [nm] = ');
lambda=1E-9*lambda;  % units conversion
omega=2*pi*cc/lambda;
k0=2*pi/lambda;
k=N*k0;
TEM=input('TE or TM mode ? (TE/TM) : ','s');
if strcmp(TEM,'te')
    TEM='TE';
end
if strcmp(TEM,'TE')
    P=N;
else
    P=1/N;
end
%
% Determine the interval of interest for ell 
%
deltap=lambda*P/(2*pi*N*sqrt(N^2-1));
ell1=-0.5+2*pi*(R+deltap)/lambda;
ell2=-0.5+2*N*pi*(R+deltap)/lambda;
if expert_mode==1
    I=input('Interval of interest for the mode indices search : ');
    ell1=I(1);
    ell2=I(2);
end
%
% Plot the modal equation function which is discontinuous
% fplot('modeqel',[ell1 ell2 -10 10],[ ],[ ],[],lambda,N,R,TEM)
hpas=(ell2-ell1)/10000;
x=ell1:hpas:ell2;
y=modeqel(x,P,k0,k,R,TEM);
plotd(x,y);
ylim([-1 1])
hold on
plot([ell1 ell2], [0 0], 'x-k','LineWidth',2); % emphasis the zero value
if strcmp(TEM,'TE')
    title('Modale function for TE modes')
else
    title('Modale function for TM modes')
end
grid
%
% Looking for all the zeros  
%
tol=1;
j=1;
for i=1:length(x)-1
    % the test ensures that we have a sign change but not a discontinuity
    if (y(i+1)*y(i)<0)&&(abs(y(i+1)-y(i))<tol)
        options = optimset('TolFun',1E-9,'TolX',1E-9);
        ell(j)=fzero(@modeqel,[x(i),x(i+1)],options,P,k0,k,R,TEM);
        j=j+1;
    end
end
disp(' ')
disp('Modes number \ell : ')
ell=fliplr(ell);
for j=1:length(ell)
    disp([num2str(j),' & ', num2str(ell(j)),'&']);
end
