function eq = modeqwv(lambda,ell,P,R,N,TEM)
%MODEQVW Modale equation for TE OR TM modes for a fixed mode index ell and 
%        a variable wavelength lambda
%
% Author: Stephane BALAC, FOTON UMR CNRS 6082, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 30 January 2014

k0=2*pi./lambda; % wave number in vaccum
k=N*k0;          % wave number in the micro-sphere
x=k*R;
y=k0*R;
if strcmp(TEM,'TE') % TE mode
    eq=besselj(ell+0.5,x)./besselj(ell-0.5,x)-...
        P*bessely(ell+0.5,y)./bessely(ell-0.5,y);
else
    eq=(P^2-1)*(1-2*ell)./(2*R*k0)+...
        P*besselj(ell-0.5,x)./besselj(ell+0.5,x)-...
        bessely(ell-0.5,y)./bessely(ell+0.5,y);
end

