%PLTMOD PLOT TE AND TM WG MODES IN A MICRO-SPHERE 
%   PLTMOD plot the whispering gallery modes in optical micro-sphere 
%
% Author: Stephane BALAC, FOTON UMR CNRS 6082, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 30 January 2014
%
% Copyright or  or Copr. Stephane BALAC (30 January 2014)
%
% This software is governed by the CeCILL-C license under French law 
% and abiding by the rules of distribution of free software.
% You can  use, modify and/ or redistribute the software under the terms of
% the CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
% URL "http://www.cecill.info". 
% 
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economic rights,  and the successive licensors  have only  limited
% liability. 
% 
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or 
% data to be ensured and,  more generally, to use and operate it in the 
% same conditions as regards security. 
% 
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL-C license and that you accept its terms.

clear 
close all
expert_mode=1;
I=sqrt(-1);
cc=299792458; % speed of light in vaccum
R=input('Micro-sphere radius [micro-m] R = ');
R=R*1e-6;
N=input('Optical index of the micro-sphere = ');

tetm=input('TE or TM  mode : ','s');
lambda=input('Wavelength [nm] = ');
lambda=1E-9*lambda;  % unit conversion
omega=2*pi*cc/lambda;
k=N*2*pi/lambda;

ell=input('Mode number ell = ');
m=input('Mode number m = ');
disp(' Plot required in an azimuthal plane phi=cste (A)');
repp=input(' or in a polar plane theta=cste (P) = ', 's');
if strcmp(repp,'a'), repp='A'; end
if strcmp(repp,'A')
    phi=input('Azimuthal angle [radian] : phi = ');
else
    theta=input('Polar angle [radian] : theta = ');
end
% Switch depending on the plot to be achieved
switch repp
    case 'A'
if expert_mode==1
    Rmin=input(' Minimum radial distance (\% R) = ');
    Rmin=Rmin*R;
    Rmax=input(' Maximal radial distance (\% R) = ');
    Rmax=Rmax*R;
    Tmin=input(' Minimum polar angle [radian] = ');
    Tmax=input(' Maximal polar angle [radian] = ');
else
    Rmin=0; Rmax=R; Tmin=0; Tmax=pi;
end
% generate meshgrid for the plots
pasR=(Rmax-Rmin)/100;
pasth=(Tmax-Tmin)/50;
[tht,rt] = meshgrid(Tmin:pasth:Tmax,Rmin:pasR:Rmax);
[X,Y] = pol2cart(tht,rt);

E=zeros(size(rt));
B=zeros(size(rt));
for i=1:size(rt,1)
    r=rt(i,1);
    for j=1:size(tht,2)
        theta=tht(1,j);
        Mhans=hansenm(ell,m,r,theta,phi,k,1);
        E(i,j)=norm(abs(Mhans));
        Nhans=hansenn(ell,m,r,theta,phi,k,1);
        B(i,j)=norm(abs(Nhans));
    end
end

if strcmp(tetm,'TE')||strcmp(tetm,'te') % TE mode
    tmp=B;
    B=abs((I*k/omega)*B);
else% TM mode
    temp=E;
    E=B;
    B=(I*k/omega)*tmp;
end
%
powr=-round(-log10(R));
X=X*10^(-powr);
Y=Y*10^(-powr);
%
% Plot the electric field
%
% Compute the magnitude order of E to scale the data
% Emax=max(max(E));
% ct=num2str(round(Emax));
% pow=length(ct)-1;
% E=E/10^pow;
contourf(X,Y,E)
h = findobj('Type','patch');
set(h,'LineStyle','none')
colorbar
axis equal
xlabel(['z = r cos(\theta) x 10^{',num2str(powr),'}'],'FontSize',14);
ylabel(['x = r sin(\theta) x 10^{',num2str(powr),'}'],'FontSize',14);
% title(['|E| x 10^{',num2str(pow),'}', ' for ell=',num2str(ell),', m=',...
%        num2str(m),', azimuth=',num2str(phi)],'FontSize',14);
 title(['|E|  for ell=',num2str(ell),', m=',...
        num2str(m),', azimuth=',num2str(phi)],'FontSize',14);
figure
%
% Plot the magnetic field
%
% Compute the magnitude order of B to scale the data
% Bmax=max(max(B));
% ct=num2str(round(Bmax));
% pow=length(ct)-1;
% B=B/10^pow;
%
contourf(X,Y,B)
h = findobj('Type','patch');
set(h,'LineStyle','none')
colorbar
axis equal
xlabel(['r cos(\theta) x 10^{',num2str(powr),'}'],'FontSize',14);
ylabel(['r sin(\theta) x 10^{',num2str(powr),'}'],'FontSize',14);
% title(['|B| x 10^{',num2str(pow),'}', ' for ell=',num2str(ell),', m=',...
%        num2str(m),', azimuth=',num2str(phi)],'FontSize',14);
title(['|B| for ell=',num2str(ell),', m=',...
       num2str(m),', azimuth=',num2str(phi)],'FontSize',14);
%----------------
    case 'P'
%----------------
if expert_mode==1
    Rmin=input(' Minimum radial distance (\% R) = ');
    Rmin=Rmin*R;
    Rmax=input(' Maximal radial distance (\% R) = ');
    Rmax=Rmax*R;
    Pmin=input(' Minimum azimutal angle = ');
    Pmax=input(' Maximal azimutal angle = ');
else
    Rmin=0; Rmax=R; Pmin=0; Pmax=2*pi;
end
% generate meshgrid for the plots
pasR=(Rmax-Rmin)/100;
pasph=(Pmax-Pmin)/50;
[pht,rt] = meshgrid(Pmin:pasph:Pmax,Rmin:pasR:Rmax);
[X,Y] = pol2cart(pht,rt);

E=zeros(size(rt));
B=zeros(size(rt));
for i=1:size(rt,1)
    r=rt(i,1);
    for j=1:size(pht,2)
        phi=pht(1,j);
        Mhans=hansenm(ell,m,r,theta,phi,k,1);
        E(i,j)=norm(abs(Mhans));
        Nhans=hansenn(ell,m,r,theta,phi,k,1);
        B(i,j)=norm(abs(Nhans));
    end
end

if strcmp(tetm,'TE')||strcmp(tetm,'te') % TE mode
    tmp=B;
    B=abs((I*k/omega)*B);
else% TM mode
    temp=E;
    E=B;
    B=(I*k/omega)*tmp;
end
%
powr=-round(-log10(R));
X=X*10^(-powr);
Y=Y*10^(-powr);
%
% Plot the electric field
%
% Compute the magnitude order of E to scale the data
% Emax=max(max(E));
% ct=num2str(round(Emax));
% pow=length(ct)-1;
% E=E/10^pow;
contourf(X,Y,E)
h = findobj('Type','patch');
set(h,'LineStyle','none')
colorbar
axis equal
xlabel(['x 10^{',num2str(powr),'}'],'FontSize',14);
ylabel(['x 10^{',num2str(powr),'}'],'FontSize',14);
title(['|E|  for ell=',num2str(ell),', m=',...
        num2str(m),', polar angle=',num2str(theta)],'FontSize',14);
figure
%
% Plot the magnetic field
%
% Compute the magnitude order of B to scale the data
% Bmax=max(max(B));
% ct=num2str(round(Bmax));
% pow=length(ct)-1;
% B=B/10^pow;
%
contourf(X,Y,B)
h = findobj('Type','patch');
set(h,'LineStyle','none')
colorbar
axis equal
xlabel([' x 10^{',num2str(powr),'}'],'FontSize',14);
ylabel([' x 10^{',num2str(powr),'}'],'FontSize',14);
% title(['|B| x 10^{',num2str(pow),'}', ' for ell=',num2str(ell),', m=',...
%        num2str(m),', azimuth=',num2str(phi)],'FontSize',14);
title(['|B| for ell=',num2str(ell),', m=',...
       num2str(m),', polar angle=',num2str(theta)],'FontSize',14);        
end