%VOLMOD WHISPERING GALLERY MODES VOLUME COMPUTATION
%   VOLMOD computes the volume of a whispering gallery TE or TM modes
%          in a micro-sphere
%
% Author: Stephane BALAC, FOTON UMR CNRS 6082, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 30 January 2014
%
% Copyright or  or Copr. Stephane BALAC (30 January 2014)
%
% This software is governed by the CeCILL-C license under French law
% and abiding by the rules of distribution of free software.
% You can  use, modify and/ or redistribute the software under the terms of
% the CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
% URL "http://www.cecill.info". 
% 
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economic rights,  and the successive licensors  have only  limited
% liability. 
% 
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or 
% data to be ensured and,  more generally, to use and operate it in the 
% same conditions as regards security. 
% 
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL-C license and that you accept its terms.

clear 
close all
I=sqrt(-1);
cc=299792458;  % Speed of light
expert_mode=0; % set to 1 for using the programm in expert mode
%
% Data reading 
%
R=input('Micro-sphere radius [micro-m] = ');
R=R*1e-6;
N=input('Optical index of the micro-sphere = ');
lambda=input('Wavelength [nm] = ');
lambda=1E-9*lambda;  % units conversion
omega=2*pi*cc/lambda;
k0=2*pi/lambda;
k=N*k0;
TEM=input('TE or TM mode ? (TE/TM) : ','s');
if strcmp(TEM,'te')
    TEM='TE';
end
ell=input('Mode number ell = ');
m=input('Mode number m = ');
if expert_mode
    L=input('Outside integration limit L = ');
else
    L=1.2*R;
end
%
% I - Computation of the un-normalized mode volume
%
if strcmp(TEM,'TE')
    Cste=ell*(ell+1)*pi/(4*k);
else
    Cste=k*ell*(ell+1)*pi/(4*omega^2);
end
% Intermediate computation of Bessel functions
YellR=bessely(ell+1/2,k0*R);
YellpR=bessely(ell+3/2,k0*R);
YellmR=bessely(ell-1/2,k0*R);
JellR=besselj(ell+1/2,k*R);
JellpR=besselj(ell+3/2,k*R);
JellmR=besselj(ell-1/2,k*R);
YellL=bessely(ell+1/2,k0*L);
YellpL=bessely(ell+3/2,k0*L);
YellmL=bessely(ell-1/2,k0*L);
voli=Cste*R^2*(JellR^2-JellmR*JellpR);
volo=JellR^2*(L^2*(YellL/YellR)^2-L^2*(YellmL/YellR)*(YellpL/YellR)-...
    R^2+R^2*(YellmR/YellR)*(YellpR/YellR));
if  strcmp(TEM,'TE')
    volo=(Cste/N^2)*volo;
else
    volo=Cste*volo;
end
vol=voli+volo;
%
% II - Computation of the normalization constant
%
% II-1) Contrubution of the radial dependency to the normalization constant
%
% initial guess fo the first zero of j_ell'
a1=(ell+1/2)+0.8086165*(ell+1/2)^(1/3)-0.236680*(ell+1/2)^(-1/3)...
    -0.20736*(ell+1/2)^(-1)+0.0233*(ell+1/2)^(-5/3);
% computes the first zero of j_ell' with fsolve
options = optimset('Display','off','TolFun',1e-10,'TolX',1e-10);
f = @(x)vshnor(x,ell);
a2=fsolve(f,a1,options);
maxr=(pi/(2*a2))*besselj(ell+1/2,a2)^2;
%
% II-1) computes the maximal value of the vector spherical harmonic X_ell^m 
%
h=pi/1000;
theta=h:h:pi-h;
y=(m^2+1)*slegend(ell,m,cos(theta)).^2+...
  (ell-m+1)*(ell+m+1)*slegend(ell+1,m,cos(theta)).^2-...
  2*sqrt((ell-m+1)*(ell+m+1))*slegend(ell,m,cos(theta)).*...
  slegend(ell+1,m,cos(theta));
if m~=0
    y=y.*(2*ell+1)./(8*pi*sin(theta).^2);
else
    y=y.*(2*ell+1)./(4*pi*sin(theta).^2);
end
maxx=max(y);
if strcmp(TEM,'TE')
% Square of the maximum valued of the electric field inside the sphere
    EMmax2=maxr*maxx;
else
% Square of the maximum valued of the magnetic induction inside the sphere
    EMmax2=maxr*maxx*(k/omega)^2;
end
% Volume of the mode
vol=vol/EMmax2;
disp([' Mode volume (in micron.m^3)= ',num2str(vol*1E18)]);
