%WVLRES COMPUTES THE WAVELENGTH AT RESONANCE
%   WVLRES computes, for a given mode index, the values of the wavelength
%   for which a resonance occurs
%
% Author: Stephane BALAC, FOTON UMR CNRS 6082, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 30 January 2014
%
% Copyright or  or Copr. Stephane BALAC (30 January 2014)
% 
% This software is governed by the CeCILL-C license under French law 
% and abiding by the rules of distribution of free software.
% You can  use, modify and/ or redistribute the software under the terms of
% the CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
% URL "http://www.cecill.info". 
% 
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economic rights,  and the successive licensors  have only  limited
% liability. 
% 
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or 
% data to be ensured and,  more generally, to use and operate it in the 
% same conditions as regards security. 
% 
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL-C license and that you accept its terms.

clear 
close all
I=sqrt(-1);
cc=299792458;  % Speed of light (m/s)
expert_mode=0; % set to 1 for using the programm in expert mode
%
% Data reading 
%
R=input('Micro-sphere radius [micro-m] = ');
R=R*1e-6; % conversion in unit m
N=input('Optical index of the micro-sphere = ');
ell=input('Mode index \ell = ');
TEM=input('TE or TM mode ? (TE/TM) : ','s');
if strcmp(TEM,'te')
    TEM='TE';
end
if strcmp(TEM,'TE')
    P=N;
else
    P=1/N;
end
%
% Determine the interval of interest for the wavelength 
%
alpha=P/(N*sqrt(N^2-1));
lambda1=2*pi*R/(ell+1/2-alpha);
lambda2=2*pi*R*N/(ell+1/2-alpha*N);
if expert_mode==1
    disp(['Wavelength intervalle of interest [nm] = [',num2str(lambda1*1E9),...
    ' , ',num2str(lambda2*1E9),']']);
end
% Plot the modal equation function which is discontinuous
%fplot('modeqwv',[lambda1 lambda2 -10 10],[ ],[ ],[],ell,P,R,N,TEM)
hpas=(lambda2-lambda1)/10000;
x=lambda1:hpas:lambda2;
y=modeqwv(x,ell,P,R,N,TEM);
xx=x*1E9;
plotd(xx,y);
ylim([-2 2])
hold on
plot(1E9*[lambda1 lambda2], [0 0], 'x-k','LineWidth',2);
if strcmp(TEM,'TE')
    title('Modale function for TE modes')
else
    title('Modale function for TM modes')
end
xlabel('\lambda [nm]')
grid
%
% Looking for all the resonance wavelengths 
%
tol=1;
j=1;
for k=1:length(x)-1
    % the test ensures that we have a sign change but not a discontinuity
    if (y(k+1)*y(k)<0)&&(abs(y(k+1)-y(k))<tol)
        options = optimset('TolFun',1E-9,'TolX',1E-9);
            Tlambda(j)=fzero(@modeqwv,[x(k),x(k+1)],options,ell,P,R,N,TEM);
            j=j+1;
    end
end
Tlambda=fliplr(Tlambda);
disp('Resonance wavelength [nm]: ')
for j=1:length(Tlambda)
    disp([num2str(j),' & ', num2str(1E9*Tlambda(j)),'&']);
end
disp(' ');
%
% Draw of the radial solution for the resonance wavelength
% 
clear y
disp('Draw the radial behavior of the mode for');
rep=input('       some of the computed wavelength? (y/n) : ','s');
if rep=='n', return; end
while rep~='n'
    j=input('Wavelength number of the mode: ');
    k=2*pi*N/Tlambda(j);
    k0=2*pi/Tlambda(j);
    r=[0.01:0.001:1];
    y=sqrt(pi./(2*k*r*R)).*besselj(ell+0.5,k*r*R);
    figure
    plot(r,y,'LineWidth',2);
    ylim([min(y) max(y)])
    hold on 
    r=[1:0.001:1.3];
    z=sqrt(pi./(2*k0*r*R)).*bessely(ell+0.5,k0*r*R);
    C=y(length(y))/z(1);
    z=C*z;
    plot(r,z,'LineWidth',2);
    xlabel('r/R');grid;
    title(['   Radial function dependency for ell = ',num2str(ell),...
        ' and \lambda [nm] = ',num2str(1E9*Tlambda(j))]); 
    rep=input('\n Draw for another wavelength? (y/n) : ','s');
end
%
% Draw of the radial solution for any wavelength
% 
if expert_mode==1 
    rep=input('\n\n Compare results to the Eikonale solution ? [y/n] : ','s');
    if strcmp(rep,'y') ||  strcmp(rep,'Y')
    % Eikonale solution
        cont='y';
        while strcmp(cont,'y') ||  strcmp(rep,'Y')
            n=input('Mode index n = ');
            tmp=ell+0.5+ ((ell+0.5)/2)^(1/3)*(1.5*pi*(n-0.25))^(2/3)...
                -P/sqrt(N^2-1);
            lambdae=(N*2*pi*R)/tmp;
            disp(['Eikonale sol. [nm] = ',num2str(lambdae*1E9)]);
            cont=input('\n Test fo an other value of n? [y/n] : ','s');
        end
    end
    rep=input('\n\n Draw the mode for a user-defined wavelengths? [y/n] : ','s');
    if rep=='n', return; end
    while rep~='n'
        lambda=input('Wavelength of the mode in [nm]: ');
        k=2*pi*N/(lambda*1E-9);
        k0=2*pi/(lambda*1E-9);
        r=[0.01:0.001:1];
        y=sqrt(pi./(2*k*r*R)).*besselj(ell+0.5,k*r*R);
        figure
        plot(r,y,'LineWidth',2);
        ylim([min(y) max(y)])
        hold on 
        r=[1:0.001:1.3];
        z=sqrt(pi./(2*k0*r*R)).*bessely(ell+0.5,k0*r*R);
        C=y(length(y))/z(1);
        z=C*z;
        plot(r,z,'LineWidth',2);
        xlabel('r/R');grid;
        title(['Radial function dependency for ell = ',num2str(ell),...
        ' and \lambda [nm] = ',num2str(lambda)]); 
        rep=input('Draw for another wavelength? (y/n) : ','s');
    end
end


